# Continuous Diffusion for Mixed-Type Tabular Data

## Install Instructions

First setup and replicate the pytorch version via

`pip install -r install_pytorch.txt`

Then install packages necessary for the synthcity package via

`pip install -r requirements_synthcity.txt`

Lastly, install the rest of the requirements via

`pip install -r requirements.txt`

## Train model

After the environment has been setup, you can run all models using one of the commands below. Note that this automatically also runs the evaluation, which can be quite costly. 
If you do not want to automatically evaluate the model as well, comment out `experiment.evaluate_generative_model(tune_catboost=True)` in `main.py`.

Outputs will appear in `results/DATA/exp_path`. For any of the commands, please replace `DATA` by one of the datasets names:
- acsincome
- adult
- bank
- beijing
- churn
- covertype
- default
- diabetes
- lending
- news
- nmes

If you want to change any of the model parameters, either change them in the corresponding config file (see `cfg_path`) or edit the relevant experiment file in `experiments/` directly.


### ARF

`python main.py DATA arf train --cfg_path=configs/arf/default.yaml --exp_path=arf`

### TVAE

`python main.py DATA tvae train --cfg_path=configs/tvae/default.yaml --exp_path=tvae`

### CTGAN

`python main.py DATA ctgan train --cfg_path=configs/ctgan/default.yaml --exp_path=ctgan`

### TabDDPM

For classification task datasets

`python main.py DATA tabddpm train --cfg_path=configs/tabddpm/default_class.yaml --exp_path=tabddpm`

For regression task datasets

`python main.py DATA tabddpm train --cfg_path=configs/tabddpm/default_reg.yaml --exp_path=tabddpm`

### CoDi

`python main.py DATA codi train --cfg_path=configs/codi/default.yaml --exp_path=codi`

### SMOTE
`python main.py DATA smote train --cfg_path=configs/smote/default.yaml --exp_path=smote`

### TabSyn
`python main.py DATA tabsyn train --cfg_path=configs/tabsyn/default.yaml --exp_path=tabsyn`


### CDTD (ours)

You can select any of our three noise schedule variants by changing the placeholder `SCHEDULE` in the commands below.

- a single noise schedule: `SCHEDULE=single`
- per type noise schedules: `SCHEDULE=bytype`
- per feature noise schedules: `SCHEDULE=all`


For classification datasets

`python main.py DATA cdtd train --cfg_path=configs/cdtd/default_class_SCHEDULE_tabddpm_mlp.yaml --exp_path=cdtd_SCHEDULE`


For regression datasets

`python main.py DATA cdtd train --cfg_path=configs/cdtd/default_reg_SCHEDULE_tabddpm_mlp.yaml --exp_path=cdtd_SCHEDULE`


## License

[MIT](https://choosealicense.com/licenses/mit/)
